/* eslint-disable no-param-reassign, no-undef, prefer-destructuring, prefer-rest-params, @typescript-eslint/no-empty-function */
// @ts-nocheck
// Analytics tracking ID.
const ANALYTICS_TRACKING_ID = 'UA-128744434-1';

/**
 * Creates a temporary global ga object and loads analytics.js.
 * Parameters elType, asyncScriptTag, and docFirstScriptTag are all used internally.
 * They could have been declared using 'var', instead they are declared as parameters to save
 * 4 bytes ('var ').
 * @param {Window}        win The global context object.
 * @param {HTMLDocument}  doc The DOM document object.
 * @param {string}        elType Must be 'script'.
 * @param {string}        analyticsSrc Protocol relative URL of the analytics.js script.
 * @param {string}        analyticsObjName Global name of analytics object. Defaults to 'ga'.
 * @param {HTMLElement}   asyncScriptTag Async script tag.
 * @param {HTMLElement}   docFirstScriptTag First script tag in document.
 */
(function(win, doc, elType, analyticsSrc, analyticsObjName, asyncScriptTag, docFirstScriptTag) {
    if (process.env && process.env.NODE_ENV && process.env.NODE_ENV === 'production') {
        win.GoogleAnalyticsObject = analyticsObjName; // Acts as a pointer to support renaming.

        // Creates an initial ga() function. The queued commands will be executed once analytics.js loads.
        win[analyticsObjName] = win[analyticsObjName] || function() {
            (win[analyticsObjName].q = win[analyticsObjName].q || []).push(arguments);
        };

        // Sets the time (as an integer) this tag was executed. Used for timing hits.
        win[analyticsObjName].l = 1 * new Date();

        // Insert the script tag asynchronously. Inserts above current tag to prevent blocking in addition to using the async attribute.
        asyncScriptTag = doc.createElement(elType);
        docFirstScriptTag = doc.getElementsByTagName(elType)[0];
        asyncScriptTag.async = 1;
        asyncScriptTag.src = analyticsSrc;
        docFirstScriptTag.parentNode.insertBefore(asyncScriptTag, docFirstScriptTag);
    }
}(window, document, 'script', 'https://ssl.google-analytics.com/analytics.js', 'acanalytics'));

if (process.env && process.env.NODE_ENV && process.env.NODE_ENV === 'production') {
// Create Analytics with Tracking ID
    acanalytics('create', `${ANALYTICS_TRACKING_ID}`, 'auto');

    // Removes failing protocol check. @see: http://stackoverflow.com/a/22152353/1958200
    acanalytics('set', 'checkProtocolTask', () => { });
    acanalytics('require', 'displayfeatures');

    // Track the settings page
    if (document.location.pathname === '/options.html') {
    // Send settings pageview analytics
        acanalytics('send', 'pageview', {
            page: '/options.html',
            title: 'ActiveCampaign Chrome-Extension Settings'
        });
    }

    // Track the chrome-extension page
    if (document.location.pathname === '/html/popup.html') {
    // Send settings chrome-extension popup analytics
        acanalytics('send', 'pageview', {
            page: '/html/popup.html',
            title: 'ActiveCampaign Chrome-Extension Popup'
        });
    }

    // Track the gmail page
    if (document.location.pathname.includes('mail')) {
    // Send settings gmail analytics
        acanalytics('send', 'pageview', {
            page: `${document.location.pathname}`,
            title: 'Gmail Page'
        });
    }
}

const Analytics = {
    /**
     * The click action event
     * @type {string}
     */
    clickEvent: 'click',

    /**
     * The submit action event
     * @type {string}
     */
    submitEvent: 'submit',

    pageViews: {
        sidebar: 'sidebar'
    },

    pageViewTitles: {
        contactEmptyView: 'contactEmptyView',
        contactProfileView: 'contactProfileView'
    },

    /**
     * Handles submit or click event actions and sends them to google analytics
     * @param {string} eventAction    Either a 'submit' or 'click' event action
     * @param {string} eventCategory  Name of the event category
     * @param {string} eventLabel     Name of the event label
     */
    handleEventsCommand(eventAction, eventCategory, eventLabel) {
        if (process.env && process.env.NODE_ENV && process.env.NODE_ENV === 'production') {
            acanalytics('send', 'pageview', {
                page: `${document.location.pathname}`,
                title: 'Events'
            });
            acanalytics('send', {
                hitType: 'event',
                eventCategory: `${eventCategory}`,
                eventAction: `${eventAction}`,
                eventLabel: `${eventLabel}`,
                nonInteraction: true
            });
        }
    },

    handlePageViewCommand(page, title) {
        if (process.env && process.env.NODE_ENV && process.env.NODE_ENV === 'production') {
            acanalytics('send', 'pageview', {
                page,
                title
            });
        }
    },

    /**
     * Handles if there's an error, it sends an exception hit to Google Analytics
     * @param {string} error  A description of the exception.
     * @param {boolean} fatal If the exception was fatal.
     */
    handleEventsException(error, fatal) {
        if (process.env && process.env.NODE_ENV && process.env.NODE_ENV === 'production') {
            let exDescription = error?.message || error;
            if (typeof error !== 'string') {
                exDescription = JSON.stringify(error);
            }
            acanalytics('send', 'exception', {
                exDescription,
                exFatal: fatal
            });
        }
    },

    sendExceptionToBG(errorMsg, fatal = false) {
        chrome.runtime.sendMessage({
            analyticsType: 'Exception',
            eventErrorMessage: errorMsg,
            eventFatalError: fatal
        });
    },

    sendEventToBG(eventType, eventName, eventDesc) {
        chrome.runtime.sendMessage({
            analyticsType: 'Event',
            eventType,
            eventName,
            eventDesc
        });
    },

    sendPageViewToBG(page, title) {
        chrome.runtime.sendMessage({
            analyticsType: 'PageView',
            page,
            title
        });
    }
};

export default Analytics;
